package com.sd.cainiao.service.impl;

import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.exception.ServiceException;
import com.sd.business.entity.express.po.ExpressDelivery;
import com.sd.business.entity.order.enums.OrderStatusEnum;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.order.vo.AssemblyDetailVo;
import com.sd.business.entity.order.vo.OrderEncasementDetailVo;
import com.sd.business.entity.order.vo.OrderEncasementVo;
import com.sd.business.service.express.ExpressDeliveryService;
import com.sd.business.service.order.IssueBillService;
import com.sd.business.service.order.OrderService;
import com.sd.cainiao.entity.bo.BasicKeyBo;
import com.sd.cainiao.entity.dto.ItemParamDto;
import com.sd.cainiao.entity.dto.TakeNumParamDto;
import com.sd.cainiao.service.IsticeService;
import com.taobao.pac.sdk.cp.PacClient;
import com.taobao.pac.sdk.cp.SendSysParams;
import com.taobao.pac.sdk.cp.dataobject.request.TMS_WAYBILL_GET.*;
import com.taobao.pac.sdk.cp.dataobject.response.TMS_WAYBILL_GET.TmsWaybillGetResponse;
import com.taobao.pac.sdk.cp.dataobject.response.TMS_WAYBILL_GET.WaybillCloudPrintResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
public class IsticeServiceImpl implements IsticeService {

    @Autowired
    private BasicKeyBo basicKeyBo;

    @Autowired
    private IssueBillService issueBillService;

    @Autowired
    private OrderService orderService;

    @Autowired
    private ExpressDeliveryService expressDeliveryService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public synchronized List<Map<String, Object>> takeNumBatch(List<Long> orderIdList) {

        if (orderIdList.size() == 0) {
            return Collections.emptyList();
        }

        List<OrderInfo> orderInfoList = orderService.list(q -> q
                .in(BaseIdPo::getId, orderIdList)
                .in(OrderInfo::getStatus, OrderStatusEnum.IN_PRODUCTION.getKey(), OrderStatusEnum.COMPLETION_PRODUCTION.getKey())
        );

        List<Long> expressDeliveryIdList = orderInfoList.stream().map(OrderInfo::getExpressDeliveryId).collect(Collectors.toList());
        List<ExpressDelivery> expressDeliveries = expressDeliveryService.listByIds(expressDeliveryIdList);
        Map<Long, ExpressDelivery> expressDeliveriesMap = expressDeliveries.stream().collect(Collectors.toMap(BaseIdPo::getId, Function.identity()));

        Map<Long, AssemblyDetailVo> map = new HashMap<>();

        for (OrderInfo orderInfo : orderInfoList) {

            ExpressDelivery expressDelivery = expressDeliveriesMap.get(orderInfo.getExpressDeliveryId());
            if (expressDelivery == null) {
                throw new ServiceException("订单 " + orderInfo.getCode() + " 没有找到快递网点信息，打印快递单失败");
            }

            AssemblyDetailVo assemblyDetailVo = issueBillService.assemblyDetail(orderInfo.getId());

            for (AssemblyDetailVo.SkuInfo skuInfo : assemblyDetailVo.getSkuInfoList()) {
                if (skuInfo.getSurplusQuantity().compareTo(BigDecimal.ZERO) > 0) {
                    throw new ServiceException("订单 " + orderInfo.getCode() + " 未包装包裹，打印快递单失败");
                }
            }

            map.put(orderInfo.getId(), assemblyDetailVo);

        }

        for (OrderInfo orderInfo : orderInfoList) {
            ExpressDelivery expressDelivery = expressDeliveriesMap.get(orderInfo.getExpressDeliveryId());

            TakeNumParamDto takeNumParam = new TakeNumParamDto();
            takeNumParam.setCpCode(expressDelivery.getExpressageCode());
            takeNumParam.setName(expressDelivery.getName());
            takeNumParam.setPhone(expressDelivery.getPhone());
            takeNumParam.setMobile(expressDelivery.getPhone());
            takeNumParam.setProvince(expressDelivery.getProvince());
            takeNumParam.setCity(expressDelivery.getCity());
            takeNumParam.setDistrict(expressDelivery.getDistrict());
            takeNumParam.setDetail(expressDelivery.getAddress());
            takeNumParam.setEmsTemplateUrl(expressDelivery.getTemplateUrl());

            takeNumParam.setReceiveName(orderInfo.getConsignee());
            takeNumParam.setReceiveMobile(orderInfo.getConsigneeNumber());
            takeNumParam.setReceiveProvince(orderInfo.getProvince());
            takeNumParam.setReceiveCity(orderInfo.getCity());
            takeNumParam.setReceiveDistrict(orderInfo.getCounty());
            takeNumParam.setReceiveDetail(orderInfo.getDetailedAddress());

            AssemblyDetailVo assemblyDetailVo = map.get(orderInfo.getId());

            for (OrderEncasementVo orderEncasementVo : assemblyDetailVo.getOrderEncasementList()) {
                List<OrderEncasementDetailVo> orderEncasementDetailList = orderEncasementVo.getOrderEncasementDetailList();
                List<ItemParamDto> itemParamList = orderEncasementDetailList.stream().map(item -> {
                    ItemParamDto itemParam = new ItemParamDto();
                    itemParam.setItemName(item.getSkuSpecName());
                    itemParam.setItemCount(item.getQuantity().intValue());
                    return itemParam;
                }).collect(Collectors.toList());
                takeNumParam.setItems(itemParamList);
                List<WaybillCloudPrintResponse> waybillCloudPrintResponses = takeNum(takeNumParam);

            }

        }

        return null;
    }

    public List<WaybillCloudPrintResponse> takeNum(TakeNumParamDto dto) {
        PacClient client = new PacClient(basicKeyBo.getAppKey(), basicKeyBo.getSecretKey(), basicKeyBo.getUrl());

        SendSysParams params = new SendSysParams();
        params.setFromCode(basicKeyBo.getToken());

        TmsWaybillGetRequest request = new TmsWaybillGetRequest();
        // 配送公司编码
        request.setCpCode(dto.getCpCode());
        // 发件人信息
        request.setSender(getSender(dto));
        // 交易订单信息
        request.setTradeOrderInfoDtos(getTradeOrderInfoDtoList(dto));

        TmsWaybillGetResponse response = client.send(request, params);

        if (response.isSuccess()) {
            return response.getWaybillCloudPrintResponseList();
        }

        throw new ServiceException(response.getErrorMsg());
    }

    /**
     * 获取交易订单信息
     */
    private List<TradeOrderInfoDto> getTradeOrderInfoDtoList(TakeNumParamDto dto) {
        List<TradeOrderInfoDto> tradeOrderInfoDtoList = new ArrayList<>();

        // 收件信息
        TradeOrderInfoDto tradeOrderInfoDto = new TradeOrderInfoDto();
        tradeOrderInfoDto.setObjectId(dto.getObjectId());
        tradeOrderInfoDto.setTemplateUrl(dto.getEmsTemplateUrl());
        tradeOrderInfoDto.setOrderInfo(getOrderInfoDto(dto));
        tradeOrderInfoDto.setPackageInfo(getPackageInfo(dto));
        tradeOrderInfoDto.setRecipient(getRecipient(dto));
        tradeOrderInfoDtoList.add(tradeOrderInfoDto);

        return tradeOrderInfoDtoList;
    }

    /**
     * 订单信息
     */
    private OrderInfoDto getOrderInfoDto(TakeNumParamDto dto) {
        OrderInfoDto orderInfoDto = new OrderInfoDto();
        orderInfoDto.setOrderChannelsType("OTHERS");
        orderInfoDto.setTradeOrderList(Collections.singletonList(dto.getOrderNum()));
        return orderInfoDto;
    }

    /**
     * 包装信息
     */
    private PackageInfoDto getPackageInfo(TakeNumParamDto dto) {
        // 商品信息
        List<Item> itemList = dto.getItems().stream().map(itemParamDto -> {
            Item item = new Item();
            item.setName(itemParamDto.getItemName());
            item.setCount(itemParamDto.getItemCount());
            return item;
        }).collect(Collectors.toList());

        PackageInfoDto packageInfoDto = new PackageInfoDto();
        packageInfoDto.setWeight(dto.getWeight());
        packageInfoDto.setVolume(dto.getVolume());
        packageInfoDto.setItems(itemList);
        return packageInfoDto;
    }

    /**
     * 获取发件人发件地址
     */
    private UserInfoDto getSender(TakeNumParamDto dto) {
        // 寄件地址，这里的地址需要是卖家订购电子面单服务时使用的订购地址，具体可以通过TMS_WAYBILL_SUBSCRIPTION_QUERY接口获取
        AddressDto sendAddress = new AddressDto();
        sendAddress.setProvince(dto.getProvince());
        sendAddress.setCity(dto.getCity());
        sendAddress.setDistrict(dto.getDistrict());
        sendAddress.setDetail(dto.getDetail());

        UserInfoDto sender = new UserInfoDto();
        sender.setName(dto.getName());
        sender.setPhone(dto.getPhone());
        sender.setMobile(dto.getMobile());
        sender.setAddress(sendAddress);
        return sender;
    }

    /**
     * 获取收件人收件地址
     */
    private UserInfoDto getRecipient(TakeNumParamDto dto) {
        AddressDto receiveAddress = new AddressDto();
        receiveAddress.setProvince(dto.getReceiveProvince());
        receiveAddress.setCity(dto.getReceiveCity());
        receiveAddress.setDistrict(dto.getReceiveDistrict());
        receiveAddress.setDetail(dto.getReceiveDetail());

        UserInfoDto receiver = new UserInfoDto();
        receiver.setName(dto.getReceiveName());
        receiver.setMobile(dto.getReceiveMobile());
        receiver.setAddress(receiveAddress);
        return receiver;
    }

}
